#ifndef FIFO_SIZE
  #define FIFO_SIZE 16
#endif

static u08 fifo[FIFO_SIZE];

static volatile u08 fifo_start = 0;
static volatile u08 fifo_end = 0;

// Store a byte into the FIFO buffer
void fifo_store(u08 byte) {
  // Store byte
  fifo[fifo_end] = byte;

  // Increase end pointer
  fifo_end++;

  // Check pointer
  if(fifo_end > (FIFO_SIZE - 1)) fifo_end = 0;

  // Increase start pointer if FIFO full  
  if(fifo_end == fifo_start) {
    fifo_start++;

    // Check start pointer
    if(fifo_start > (FIFO_SIZE - 1)) fifo_start = 0;
  }
}

// Load a byte from the FIFO buffer
u08 fifo_load(u08 *byte) {
  // Check size
  if(fifo_end == fifo_start) return(1);

  // Load byte
  *byte = fifo[fifo_start];

  // Increase pointer
  fifo_start++;

  // Check pointer
  if(fifo_start > (FIFO_SIZE - 1)) fifo_start = 0;

  return(0);
}
